@@ -2,13 +2,12 @@
 """ handle request for markdown pages """
 import logging
 import os
-import sys
-import subprocess
+import importlib
 from tornado.web import RequestHandler, HTTPError
 from tornado.escape import url_escape
-from .utils.converter_mixin import ConverterMixin
+from ..utils.converter_mixin import ConverterMixin
 from .access_control import UserMixin
-from .utils.nav import nav
+from ..utils.nav import nav
 
 
 LOGGER = logging.getLogger(__name__)
@@ -20,18 +19,18 @@ class SiteHandler(
 ):  # pylint: disable=W0223
     """ inline transform request for markdown pages """
 
-    def initialize(self, pages, s3_loader):
+    def initialize(self, pages):
         """ setup init properties """
         self.pages = pages
         self.meta = None
         self.nav = None
         self.site_nav = None
-        self._s3_loader = s3_loader
+        self.site = None
 
     def create_template_loader(self, template_path):
-        """ return s3 loader """
-        if self._s3_loader:
-            return self._s3_loader
+        """ if we have one, us it """
+        if self.site.template_loader:
+            return self.site.template_loader
         return super().create_template_loader(template_path)
 
     @property
@@ -48,53 +47,65 @@ class SiteHandler(
         result = self.meta_value(name, default)
         return result[0] if result else None
 
-    def load_site_nav(self, path):
+    def load_site_nav(self, site, path):
         """ set the handler site_nav attribute """
-        menu = nav(self.pages, path)
+        menu = nav(site, root=self.pages, path=path)
         if menu:
             self.site_nav = "\n".join(menu)
 
-    def load_dir_nav(self, path):
+    def load_dir_nav(self, site, path):
         """ load nav section if it exist """
         folder = os.path.dirname(path)
         if folder:
             LOGGER.info(" -- folder: %s", folder)
             nav_path = os.path.join(folder, "-nav.md")
-            if os.path.isfile(nav_path):
+            _, content = site.get_file(nav_path)
+            if content:
+                content = content.decode("utf-8")
                 LOGGER.info(" -- nav: %s", nav_path)
-                with open(nav_path, "r", encoding="utf-8") as file:
-                    content = self.meta.convert(file.read())
-                    self.nav = self.convert_images(content)
+                content = self.meta.convert(content)
+                self.nav = self.convert_images(content)
+
+    def run_script(
+        self, site, script_name, path
+    ):  # pylint: disable=unused-argument
+        """ load a module and call module.main """
+        name = f"{self.application.settings['script_path']}.{script_name}"
+        script_module = importlib.import_module(name)
+
+        return script_module.main(path)
 
     async def get(self, path):
         """ handle get """
         path = path if path else "index.html"
+
         file, ext = os.path.splitext(path)
 
         doc = os.path.join(self.pages, f"{file}.md")
-        if not os.path.isfile(doc):
+        self.site = self.get_site(path)
+        _, content = self.site.get_file(doc)
+        if content is None:
             raise HTTPError(404)
+        if content:
+            content = content.decode("utf-8")
 
         self.meta = self.markdown
-        self.load_dir_nav(doc)
-        self.load_site_nav(path)
+        self.load_dir_nav(self.site, doc)
+        self.load_site_nav(self.site, path)
 
         file_path = os.path.split(file)[0]
 
         # load theme
         theme_file = os.path.join(self.pages, file_path, "-theme.css")
-        theme_css = None
-        if os.path.isfile(theme_file):
-            theme_css = open(theme_file).read()
+        _, theme_css = self.site.get_file(theme_file)
+        if theme_css:
             LOGGER.info(" -- theme.css")
+            theme_css = theme_css.decode("utf-8")
 
         edit_path = "/edit"
         if file:
             edit_path = f"/edit?path={ url_escape(file) }.md"
 
-        with open(doc, "r", encoding="utf-8") as file:
-            content = file.read()
-
         LOGGER.info(" -- ext: %s", ext)
         if ext == ".html":
             content = self.meta.convert(content)
@@ -102,23 +113,9 @@ class SiteHandler(
             template = self.one_meta_value("template", "site")
             LOGGER.info(" -- tmpl: %s", template)
             for key in self.meta.Meta:
-                if key.startswith("x-cgi-"):
-                    LOGGER.info(" -- cgi %s", key)
-                    cgi_script = os.path.abspath(
-                        f"{self.application.settings['duck_cgi']}/{self.meta.Meta[key][0]}.py"
-                    )
-                    LOGGER.info(" -- cgi %r", cgi_script)
-                    env = {
-                        "REQUEST_URI": path,
-                        "PATH": os.getcwd(),
-                        "CONTENT_LENGTH": "-1",
-                        "SCRIPT_FILENAME": cgi_script,
-                    }
-                    outcome = subprocess.run(
-                        [sys.executable, cgi_script],
-                        env=env,
-                        capture_output=True,
-                        check=False,
+                if key.startswith("x-script-"):
+                    outcome = self.run_script(
+                        self.site, self.meta.Meta[key][0], path
                     )
                     self.meta.Meta[key] = [outcome]
             self.render(
